library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity ENC is port(
    RES  : in    std_logic;  -- system reset, low active
    CLK  : in    std_logic;  -- 24MHz for 4Mbps
    A,B  : in    std_logic;  -- rotary encoder pulse
    SCKX : in    std_logic_vector(2 downto 0);  -- prescaler
    DIR  : out   std_logic;  -- direction of encoder
    EDG  : out   std_logic); -- encoder pulse edge
end ENC;

architecture RTL of ENC is
signal C : std_logic := '0';
signal E : std_logic := '0';  -- EDG
signal Q : std_logic := '0';
signal D : std_logic := '0';  -- DIR

begin
process (RES,CLK) begin
    if (RES = '0') then
                C <= '0';  E <= '0';
    elsif (CLK'event and CLK = '1') then
        if (SCKX /= "011") then
                C <= C;    E <= E;
        elsif (C = '0') and (E = '0') then
            if (A /= B) then
                C <= '1';  E <= '1';
            else
                C <= '0';  E <= '0';
            end if;
        elsif (C = '1') and (E = '1') then
                C <= '1';  E <= '0';
        elsif (C = '1') and (E = '0') then
            if (A /= B) then
                C <= '1';  E <= '0';
            else
                C <= '0';  E <= '1';
            end if;
        else  -- "01"
                C <= '0';  E <= '0';
        end if;
    end if;
end process;
    EDG <= E;
process (RES,CLK) begin
    if (RES = '0') then
                Q <= '0';
    elsif (CLK'event and CLK = '1') then
        if (SCKX = "011") then
                Q <= A;
        else
                Q <= Q;
        end if;
    end if;
end process;
process (RES,CLK) begin
    if (RES = '0') then
                    D <= '0';
    elsif (CLK'event and CLK = '1') then
        if (SCKX = "011") then
            if (A xor B xor C) = '1' then
                if (A /= Q) then
                    D <= A xor B;
                else -- (A = Q)
                    D <= A xnor B;
                end if;
            else
                    D <= D;
            end if;
        else
                    D <= D;
        end if;
    end if;
end process;
    DIR <= D;
end RTL;
