library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity RXC is port(
    RES  : in    std_logic;  -- system reset, low active
    DOUT : out   std_logic_vector(3 downto 0);  -- data bus
    CS   : in    std_logic;  -- chip select
    RD   : in    std_logic;  -- write enable
    CLK  : in    std_logic;  -- 24MHz for 4Mbps
    SCKX : in    std_logic_vector(2 downto 0);  -- prescaler
    SCI  : in    std_logic_vector(4 downto 0);
    STEP : in    std_logic;  -- motor step command
    SD   : in    std_logic);  -- serial data input
end RXC;

architecture RTL of RXC is
signal SCK2,SCK3,SCK5,SCK6 : std_logic := '0';
signal SSD     : std_logic := '0';  -- synchronized SD input
signal RSD     : std_logic := '0';  -- recieved serial data
signal RDR     : std_logic_vector(3 downto 0) := "0000";
signal DOUT_fb : std_logic_vector(3 downto 0) := "0000";

begin
-- SCI clocks -------------------------------------------------
    SCK2 <= '1' when (SCKX = "010") else '0';
    SCK3 <= '1' when (SCKX = "011") else '0';  -- bit boundary
    SCK5 <= '1' when (SCKX = "101") else '0';
    SCK6 <= '1' when (SCKX = "110") else '0';  -- center of bit cell

-- FSK demodulator --------------------------------------------
process (RES,CLK) begin
    if (RES = '0') then
            SSD <= '0';
    elsif (CLK'event and CLK ='1') then
        if (SCK5 = '1') then
            SSD <= SD;
        else
            SSD <= SSD;
        end if;
    end if;
end process;
process (RES,CLK) begin
    if (RES = '0') then
            RSD <= '0';
    elsif (CLK'event and CLK ='1') then
        if (SCK2 = '1') then
            RSD <= SSD xor SD;
        else
            RSD <= RSD;
        end if;
    end if;
end process;

-- recieve data register --------------------------------------
process (RES,CLK) begin
    if (RES = '0') then
        RDR <= "0000";
    elsif (CLK'event and CLK = '1') then
        if (SCK3 = '1') then
            case SCI is
                when "11100" =>  RDR(0) <= RSD;  -- SE(0)
                when "11101" =>  RDR(1) <= RSD;  -- SE(1)
                when "11110" =>  RDR(2) <= RSD;  -- SE(2)
                when "11111" =>  RDR(3) <= RSD;  -- SE(3)
                when others  =>  RDR    <= RDR;
            end case;
        else
            RDR <= RDR;
        end if;
    end if;
end process;

-- output data bus --------------------------------------------
process (RES,CLK) begin
    if (RES = '0') then
            DOUT_fb <= "0000";
    elsif (CLK'event and CLK = '1') then
        if (SCK6 = '1') and (SCI = "11111") then
            DOUT_fb <= RDR;
        else
            DOUT_fb <= DOUT_fb;
        end if;
    end if;
end process;
    DOUT <= "ZZZZ" when (CS = '1') else
            "ZZZZ" when (RD = '1') else DOUT_fb;
end RTL;
