library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity SCI is port(
    RES  : in    std_logic;  -- system reset, low active
    CLK  : in    std_logic;  -- 24MHz for 4Mbps
    SCKX : in    std_logic_vector(2 downto 0);  -- prescaler
    PLS  : in    std_logic;  -- motor step pulse input
    EDG  : in    std_logic;  -- encoder pulse edge
    MODE : in    std_logic;  -- select motor step pulse input
    STEP : inout std_logic;  -- motor step command
    SCI  : inout std_logic_vector(4 downto 0));
end SCI;

architecture RTL of SCI is
signal SCK6 : std_logic := '0';  -- center of bit cell
signal MSP  : std_logic := '0';  -- synchronized motor step pulse
signal TMR  : std_logic_vector(5 downto 0) := "000000";
signal IDLE,BUSY,FIN : std_logic := '0';  -- state of TMR

begin
-- SCI clocks -------------------------------------------------
    SCK6 <= '1' when (SCKX = "110") else '0';  -- center of bit cell

-- Serial Communication Interface controler -------------------
process (RES,CLK) begin
    if (RES = '0') then                  -- grobal reset line
                    SCI <= "00000";      -- first stop bit
    elsif (CLK'event and CLK = '1') then -- grobal clock line
        if (SCK6 = '1') then             -- clock enable PT
            if  (SCI(4) = '0') then      -- stop bit
                if (STEP = '1') then     -- mortor step command
                    SCI <= "10000";      -- restart
                elsif (BUSY = '1') then  -- timer is running
                    SCI <= SCI;          -- wait
                else  -- IDLE or FIN
                    SCI <= SCI + '1';    -- free running
                end if;
            else -- SCI(4) = '1'
                    SCI <= SCI + '1';    -- transmit
            end if;
        else     -- SCK6 = '0'
                    SCI <= SCI;
        end if;
    end if;
end process;

-- PLS synchronizer -------------------------------------------
process (RES,CLK) begin
    if (RES = '0') then
                MSP <= '0';
    elsif (CLK'event and CLK = '1') then
        if (SCK6 = '1') then
            if (MODE = '0') then
                MSP <= PLS;
            else
                MSP <= EDG;
            end if;
        else  -- SCK6 = '0'
                MSP <= MSP;
        end if;
    end if;
end process;

-- wait timer for motor step command --------------------------
process (RES,CLK) begin
    if (RES = '0') then
                    TMR <= "000000";     -- idle state
    elsif (CLK'event and CLK = '1') then
        if (SCK6 = '1') then
            if (IDLE = '1') then         -- idle
                if (MSP = '1') then      -- synchronized PLS
                    TMR <= "010000";     -- timer start
                else
                    TMR <= TMR;
                end if;
            elsif (BUSY = '1') then      -- running
                    TMR <= TMR + '1';
            elsif (STEP = '1') then      -- transmit start
                    TMR <= TMR + '1';
            else  -- (FIN = '1')
                if (MSP = '0') then
                    TMR <= "000000";     -- return to idle
                else
                    TMR <= TMR;
                end if;
            end if;
        else  -- SCK6 = '0'
                    TMR <= TMR;
        end if;
    end if;
end process;
    IDLE <= '1' when (TMR(5 downto 4) = "00") else '0';
    BUSY <= '1' when (TMR(5 downto 4) = "01") else '0';
    STEP <= '1' when (TMR(5 downto 4) = "10") else '0';
    FIN  <= '1' when (TMR(5 downto 4) = "11") else '0';

end RTL;
